@@ -1,5 +1,5 @@
 # Arduino python implementation
-# Copyright (C) 2010  Lucian Langa <cooly@gnome.eu.org>
+# Copyright (C) 2010-2012  Lucian Langa <cooly@gnome.eu.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,6 @@
 
 import os
 import sys
-import ConfigParser
 import config
 import prefs
 import misc
@@ -33,22 +32,36 @@ class Board(object):
 		self.boards = []
 		self.programmers = []
 		self.defaults = []
-		conf = ConfigParser.RawConfigParser()
-		path = misc.getArduinoFile("BOARDS")
-		if path is None: raise SystemExit(_("Cannot load BOARDS file. Exiting."))
-		conf.read(path)
-		if not len(conf.sections()):
-			raise SystemExit(_("Error reading %s file. File is corrupt. Installation problem.\n") % path)
-		c = 1
-		for i in conf.sections():
-			v = dict(conf.items(i))
-			v['id'] = c
-			v['desc'] = i
-			self.boards.append(v)
-			c = c + 1
+		try:
+			self.boards.extend(self.readCustomBoards())
+		except: pass
+		try:
+			self.boards.extend(self.readArduinoBoards())
+		except:
+			self.boards.extend(misc.readGnoduinoConfFile("BOARDS"))
+		#renumber ids in case we grown with customs
+		for i in range(len(self.boards)): self.boards[i]['id'] = i+1
 		self.p = prefs.preferences()
 		if config.cur_board == -1:
-			config.cur_board = self.getBoardIdByName(self.p.getSafeValue("board", "uno")) - 1
+			"""error can occur when placing different types of hardware in sketchdir"""
+			try:
+				config.cur_board = self.getBoardIdByName(self.p.getSafeValue("board", "uno")) - 1
+			except:
+				config.cur_board = self.getBoardIdByName("uno") - 1
+
+	def readArduinoBoards(self):
+		return misc.readArduinoConfFile(misc.getArduinoFile("hardware/arduino/boards.txt"))
+
+	def readCustomBoards(self):
+		custom = []
+		p = prefs.preferences()
+		d = os.path.join(p.getValue("sketchbook.path"), "hardware")
+		try:
+			for i in os.listdir(d):
+				if os.path.exists(os.path.join(d, i, "boards.txt")):
+					custom.extend(misc.readArduinoConfFile(os.path.join(d, i, "boards.txt")))
+			return custom
+		except:	return None
 
 	def getBoards(self):
 		return self.boards
@@ -66,10 +79,14 @@ class Board(object):
 		return self.boards[id]['f_cpu']
 
 	def getPGM(self, id):
-		return self.boards[id]['protocol']
+		try:
+			return self.boards[id]['protocol']
+		except KeyError: return ""
 
 	def getPGMSpeed(self, id):
-		return self.boards[id]['speed']
+		try:
+			return self.boards[id]['speed']
+		except KeyError: return 0
 
 	def getFuseLock(self, id):
 		return self.boards[id]['lock_bits']
@@ -84,11 +101,23 @@ class Board(object):
 		return self.boards[id]['low_fuses']
 
 	def getFuseExtended(self, id):
-		return self.boards[id]['extended_fuses']
+		try:
+			return self.boards[id]['extended_fuses']
+		except KeyError: return ""
 
 	def getPath(self, id):
 		return self.boards[id]['path']
 
+	def getVariant(self, id):
+		try:
+			return self.boards[id]['variant']
+		except KeyError: return ""
+
+	def getHardwarePath(self, id):
+		try:
+			return self.boards[id]['hwpath']
+		except KeyError: return ""
+
 	def getBootloader(self, id):
 		return self.boards[id]['file']
 
