@@ -1,5 +1,5 @@
 # Arduino python implementation
-# Copyright (C) 2010  Lucian Langa <cooly@gnome.eu.org>
+# Copyright (C) 2010-2012  Lucian Langa <cooly@gnome.eu.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -48,12 +48,14 @@ cobj = [
 	"wiring_shift.c",
 	"wiring_analog.c",
 	"wiring_pulse.c",
-#	"wiring_serial.c",
-	"pins_arduino.c",
 	"WInterrupts.c",
 	"wiring.c"
 	]
 
+cobj_deprecated = [
+	"pins_arduino.c"
+	]
+
 cppobj = [
 	"WMath.cpp",
 	"HardwareSerial.cpp",
@@ -62,6 +64,15 @@ cppobj = [
 	"Tone.cpp"
 	]
 
+cppobj_additional = [
+			"new.cpp",
+			"WString.cpp",
+			"CDC.cpp",
+			"Stream.cpp",
+			"IPAddress.cpp",
+			"USBCore.cpp"
+			]
+
 defc = [
 	"/usr/bin/avr-gcc",
 	"-c",
@@ -129,6 +140,8 @@ def compile(tw, id, output, notify):
 	tmpdir = id
 	tempobj = tempfile.mktemp("", "Tempobj", id)
 	global p
+	global queue
+	queue = []
 	p = prefs.preferences()
 	b = board.Board()
 	#compile inter c objects
@@ -137,43 +150,37 @@ def compile(tw, id, output, notify):
 		(pre_file, lines) = preproc.addHeaders(id, buf)
 		"""compile C targets"""
 		misc.printLogMessageLn('processing C targets')
-		for i in cobj:
-			compline=[j for j in defc]
-			compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
-			compline.append("-DF_CPU="+b.getBoardFCPU(b.getBoard()))
-			compline.append("-I"+misc.getArduinoPath())
-			compline.append(os.path.join(misc.getArduinoPath(), i))
-			compline.append("-o"+id+"/"+i+".o")
-			misc.printMessageLn(output, ' '.join(compline))
-			misc.printLogMessageLn(' '.join(compline))
-			(run, sout) = misc.runProg(compline)
-			misc.printLogMessageLn(sout)
+		(run, sout) = compileObjects(cobj, defc, id, output)
+		if run == False:
+			misc.printErrorLn(notify, output, _("Compile Error"), stripOut(sout, pre_file))
+			raise NameError("compile error")
+		"""deprecated C targets (pre 1.0)"""
+		if misc.getArduinoVersion() < 100:
+			(run, sout) = compileObjects(cobj_deprecated, defc, id, output)
 			if run == False:
 				misc.printErrorLn(notify, output, _("Compile Error"), stripOut(sout, pre_file))
 				raise NameError("compile error")
-			else:
-				misc.printMessageLn(output, sout)
 		"""compile C++ targets"""
 		misc.printLogMessageLn('processing C++ targets')
-		for i in cppobj:
-			compline = [j for j in defcpp]
-			compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
-			compline.append("-DF_CPU="+b.getBoardFCPU(b.getBoard()))
-			compline.append("-I" + misc.getArduinoPath())
-			compline.append(os.path.join(misc.getArduinoPath(), i))
-			compline.append("-o"+id+"/"+i+".o")
-			misc.printMessageLn(output, ' '.join(compline))
-			misc.printLogMessageLn(' '.join(compline))
-			(run, sout) = misc.runProg(compline)
-			misc.printLogMessageLn(sout)
+		(run, sout) = compileObjects(cppobj, defcpp, id, output)
+		if run == False:
+			misc.printErrorLn(notify, output, _("Compile Error"), sout)
+			raise NameError("compile error")
+		"""compile C++ additional (1.0) targets"""
+		if misc.getArduinoVersion() >= 100:
+			misc.printLogMessageLn('processing C++ additional targets')
+			(run, sout) = compileObjects(cppobj_additional, defcpp, id, output)
 			if run == False:
 				misc.printErrorLn(notify, output, _("Compile Error"), sout)
 				raise NameError("compile error")
-			else:
-				misc.printMessageLn(output, sout)
 		"""generate archive objects"""
 		misc.printLogMessageLn('generating ar objects')
-		for i in cobj+cppobj:
+		objects = cobj + cppobj
+		if misc.getArduinoVersion() >= 100:
+			objects += cppobj_additional
+		else:
+			objects += cobj_deprecated
+		for i in objects:
 			compline = [j for j in defar]
 			compline.append(id+"/core.a")
 			compline.append(id+"/"+i+".o")
@@ -192,11 +199,14 @@ def compile(tw, id, output, notify):
 		compline=[j for j in defcpp]
 		compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
 		compline.append("-DF_CPU="+b.getBoardFCPU(b.getBoard()))
-		compline.append("-I" + misc.getArduinoPath())
+		if misc.getArduinoVersion() >= 100:
+			compline.append("-DARDUINO=100")
+		compline.extend(misc.getArduinoIncludes())
 		flags = []
 		flags = preproc.generateCFlags(id, cont)
 		compline.extend(flags)
 		compline.extend(["-I" + os.path.join(i, "utility") for i in preproc.generateLibs(id, buf)])
+		compline.extend(misc.getArduinoIncludes())
 		compline.append(pre_file)
 		compline.append("-o"+pre_file+".o")
 		misc.printMessageLn(output, ' '.join(compline)+"\n")
@@ -221,7 +231,11 @@ def compile(tw, id, output, notify):
 		for i in preproc.generateLibs(id, buf):
 			tmplibs.extend(validateLib(os.path.basename(i), tempobj, flags, output, notify))
 		compline.extend(list(set(tmplibs)))
+		compline.extend(["-I" + i for i in preproc.generateLibs(id, buf)])
 		compline.extend(["-I" + os.path.join(i, "utility") for i in preproc.generateLibs(id, buf)])
+		if misc.getArduinoVersion() >= 100:
+			compline.append("-DARDUINO=100")
+		compline.extend(misc.getArduinoIncludes())
 		compline.append(id+"/core.a")
 		compline.append("-L"+id)
 		compline.append("-lm")
@@ -261,7 +275,7 @@ def compile(tw, id, output, notify):
 			misc.printMessageLn(output, sout)
 		size = computeSize(tempobj+".hex")
 		notify.pop(context)
-		notify.push(context, _("Done compilling."))
+		notify.push(context, _("Done compiling."))
 		misc.printLogMessageLn("compile done.")
 
 		misc.printMessageLn(output, \
@@ -275,6 +289,29 @@ def compile(tw, id, output, notify):
 		return -1
 	return tempobj
 
+def compileObjects(objects, flags, objout, output):
+	status = True
+	sout = ""
+	b = board.Board()
+	for i in objects:
+		compline = [j for j in flags]
+		compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
+		compline.append("-DF_CPU="+b.getBoardFCPU(b.getBoard()))
+		compline.extend(misc.getArduinoIncludes())
+		if misc.getArduinoVersion() >= 100:
+			compline.append("-DARDUINO=100")
+		compline.append(os.path.join(misc.getArduinoPath(), i))
+		compline.append("-o"+objout+"/"+i+".o")
+		misc.printMessageLn(output, ' '.join(compline))
+		misc.printLogMessageLn(' '.join(compline))
+		try:
+			(status, sout) = misc.runProg(compline)
+			misc.printLogMessageLn(sout)
+			misc.printMessageLn(output, sout)
+		except:
+			return (status, sout)
+	return (True, "")
+
 def getLibraries():
 	paths = ["", misc.getArduinoPath()]
 	dirs = ["", "utility"]
@@ -289,17 +326,19 @@ def getLibraries():
 def validateLib(library, tempobj, flags, output, notify):
 	"""compile library also try to compile every cpp under libdir"""
 	"""also try to compile utility dir if present"""
-	paths = ["", misc.getArduinoPath(), misc.getArduinoLibsPath()]
+	paths = ["", misc.getArduinoLibsPath()]
 	if config.user_library != None and config.user_library != -1:
 		paths.extend(i.strip() for i in config.user_library.split(';'))
 	dirs = ["", "utility"]
 	b = board.Board()
 	res = []
 	fl = []
-	queue = []
 	for d in dirs:
 		for q in paths:
 			fl = os.path.join(q, library, d)
+			"""test the path directly, in case include was specified to the library directory"""
+			if fl not in set(queue) and not os.path.exists(fl):
+				fl = os.path.join(q, d)
 			if fl not in set(queue) and os.path.exists(fl):
 				queue.append(fl)
 				try:
@@ -308,7 +347,9 @@ def validateLib(library, tempobj, flags, output, notify):
 						compline = [j for j in defc]
 						compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
 						compline.append("-DF_CPU="+b.getBoardFCPU(b.getBoard()))
-						compline.append("-I" + misc.getArduinoPath())
+						if misc.getArduinoVersion() >= 100:
+							compline.append("-DARDUINO=100")
+						compline.extend(misc.getArduinoIncludes())
 						compline.append("-I" + os.path.join(misc.getArduinoLibsPath(), library, "utility"))
 						compline.extend(preproc.generateCFlags(id, open(i).read()))
 						compline.extend(flags)
@@ -319,7 +360,7 @@ def validateLib(library, tempobj, flags, output, notify):
 						(run, sout) = misc.runProg(compline)
 						misc.printLogMessageLn(sout)
 						if run == False:
-							misc.printError(notify, output, _("Library Error"), sout)
+							misc.printErrorLn(notify, output, _("Library Error"), sout)
 							raise NameError('libs compile error')
 						res.append(os.path.join(os.path.dirname(tempobj), \
 							os.path.basename(i.replace(".c", ".o"))))
@@ -331,7 +372,9 @@ def validateLib(library, tempobj, flags, output, notify):
 						compline = [j for j in defcpp]
 						compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
 						compline.append("-DF_CPU="+b.getBoardFCPU(b.getBoard()))
-						compline.append("-I" + misc.getArduinoPath())
+						if misc.getArduinoVersion() >= 100:
+							compline.append("-DARDUINO=100")
+						compline.extend(misc.getArduinoIncludes())
 						compline.extend(preproc.generateCFlags(id, open(i).read()))
 						compline.extend(flags)
 						compline.append(i)
@@ -342,7 +385,7 @@ def validateLib(library, tempobj, flags, output, notify):
 						(run, sout) = misc.runProg(compline)
 						misc.printLogMessageLn(sout)
 						if run == False:
-							misc.printError(notify, output, _("Library Error"), sout)
+							misc.printErrorLn(notify, output, _("Library Error"), sout)
 							raise NameError('libs compile error')
 						res.append(os.path.join(os.path.dirname(tempobj), \
 							os.path.basename(i.replace(".cpp", ".o"))))
