@@ -2,10 +2,13 @@
 #define current selected board
 cur_board = -1
 cur_serial_port = -1
+cur_programmer = -1
 cur_iter = -1
 cur_editor_font = -1
 cur_console_font = -1
 build_verbose = -1
+upload_verbose = -1
 user_library = -1
 serial_baud_rate = -1
 show_numbers = -1
+sketchdir = -1
