@@ -23,7 +23,6 @@ import gettext
 _ = gettext.gettext
 
 import gtksourceview2
-import gnomevfs
 import misc
 import prefs
 import time
@@ -44,12 +43,7 @@ def get_lang_for_file(f):
 				return lang
 
 def get_lang_for_content(content):
-	mime = gnomevfs.get_mime_type_for_data(content)
-	"""try harder to guess mime type"""
-	"""sometimes helps to trim leading newlines"""
-	if mime == "text/plain":
-		tmpmime = gnomevfs.get_mime_type_for_data(content.strip("\n"))
-		if mime is not None: mime = tmpmime
+	mime = misc.get_mime_type(content)
 	manager = gtksourceview2.language_manager_get_default()
 	languages = gtksourceview2.LanguageManager.get_language_ids(manager)
 	for l in languages:
@@ -62,7 +56,8 @@ def get_lang_for_content(content):
 				return lang
 
 def instext(b, iter, text, len) :
-	lang = get_lang_for_content(text)
+	cont = b.get_text(b.get_start_iter(), b.get_end_iter())
+	lang = get_lang_for_content(cont)
 	if lang != None:
 		b.set_language(lang)
 
