@@ -24,8 +24,10 @@ import subprocess
 import select
 import shutil
 import signal
-import time
+import urlparse
+import urllib
 
+import locale
 import gettext
 _ = gettext.gettext
 
@@ -40,8 +42,8 @@ import serialio
 import srcview
 import gnoduino
 
-import gtksourceview2
 font = "Monospace 10"
+APP_NAME = "gnoduino"
 
 def setupPage(w, page, p):
 	misc.set_widget_font(getCurrentView(), config.cur_editor_font)
@@ -61,9 +63,9 @@ def destroyPage(w, b):
 	f = b.get_data("file")
 	buf = b.get_data("buffer")
 	if misc.bufferModified(buf, f) is True:
-		if f is None: f= "Untitled"
+		if f is None: f = _("Untitled")
 		save = misc.createPopup(_("Save document"), mainwin, \
-			_("Save changes to document \"%s\"\n before closing?" % os.path.basename(f)))
+			_("Save changes to document \"%s\"\n before closing?") % os.path.basename(f))
 		if save == gtk.RESPONSE_YES:
 			if csave(None, False) is False:
 				return False
@@ -81,8 +83,7 @@ def updatePageTitle(w, status):
 	if f != None:
 		name = os.path.basename(f)
 	else:
-		fh = 0
-		name = "Untitled"
+		name = _("Untitled")
 
 	if (misc.bufferModified(w, f)):
 		l.set_text("*"+name)
@@ -95,7 +96,7 @@ def switchPage(page, a, b, c):
 
 def createPage(nb, f=None):
 	hbox = gtk.HBox(False, 0)
-	flabel = gtk.Label(os.path.basename(f) if f else "Untitled")
+	flabel = gtk.Label(os.path.basename(f) if f else _("Untitled"))
 	hbox.pack_start(flabel, True, False, 3)
 	b = gtk.Button()
 	img = gtk.Image()
@@ -153,31 +154,32 @@ def openFile(w):
 	if searchFile(nb, f) == True:
 		w.destroy()
 		return
+	addRecentItem(recentmanager, f)
 	processFile(f)
 
 def processFile(f):
 	if not os.path.exists(f):
 		return
+	if searchFile(nb, f) == True:
+		return
 	page = getCurrentPage()
 	createPage(nb, f)
-	if page.get_data("label").get_text() == "Untitled":
+	if page.get_data("label").get_text() == _("Untitled"):
 		replacePage(page)
 
 def saveAs(js=False):
 	page = getCurrentPage()
 	buf = page.get_data("buffer")
-	f = page.get_data("file")
-	if misc.bufferModified(buf, f) is False and js is False:
+	cur_file = page.get_data("file")
+	if misc.bufferModified(buf, cur_file) is False and js is False:
 		return
-	p = gtk.FileChooserDialog("Save file", None, gtk.FILE_CHOOSER_ACTION_SAVE,
+	p = gtk.FileChooserDialog(_("Save file"), None, gtk.FILE_CHOOSER_ACTION_SAVE,
 		(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
 		gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-	if page.get_data("file") is not None:
-		name =  os.path.basename(page.get_data("file"))
-		p.set_filename(name)
+	if cur_file is not None:
+		p.set_filename(cur_file)
 	else:
-		name = "Untitled"
-		p.set_current_name(name)
+		p.set_current_name(_("Untitled"))
 	p.set_default_size(450, 400)
 	p.show_all()
 	if p.run() == gtk.RESPONSE_ACCEPT:
@@ -188,9 +190,12 @@ def saveAs(js=False):
 	return None
 
 def copen(widget, data=None):
-	p = gtk.FileChooserDialog("Open file", None, gtk.FILE_CHOOSER_ACTION_OPEN,
+	p = gtk.FileChooserDialog(_("Open file"), None, gtk.FILE_CHOOSER_ACTION_OPEN,
 		(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
 		gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+	cur_file = getCurrentPage().get_data("file")
+	if cur_file is not None:
+		p.set_current_folder(os.path.dirname(cur_file))
 	p.set_size_request(450, 400)
 	p.show_all()
 	if p.run() == gtk.RESPONSE_ACCEPT:
@@ -211,14 +216,15 @@ def csave(w, data=None):
 		else:
 			if os.path.exists(f):
 				p = misc.MessageBox()
-				res = p.show("""<b>A file named %s already exists. Do you want to replace it?</b>""" % f,
-				"The file already exists in \"%s\". Replacing it will overwrite its contents." % os.path.dirname(f))
+				res = p.show(_("""<b>A file named %s already exists. Do you want to replace it?</b>""") % f,
+				_("The file already exists in \"%s\". Replacing it will overwrite its contents.") % os.path.dirname(f))
 				if not res: return
 			l.set_text(os.path.basename(f))
 			page.set_data("file", f)
 	F = open(f, "w")
 	F.write(b.get_text(b.get_start_iter(), b.get_end_iter()))
 	F.close()
+	addRecentItem(recentmanager, f)
 	updatePageTitle(b, sb2)
 	return True
 
@@ -247,7 +253,9 @@ def libImport(widget, data=None):
 def compile(widget, data=file):
 	cserial(None, 0, sctw)
 	page = getCurrentPage()
+	startSpinner()
 	obj = compiler.compile(page.get_data("view"), id, tw, sb) #page.get_data("buffer")
+	stopSpinner()
 	return obj
 
 def upload(widget, serial, data=file):
@@ -255,7 +263,9 @@ def upload(widget, serial, data=file):
 	if obj == -1: return
 	while (gtk.events_pending()):
 		gtk.main_iteration()
+	startSpinner()
 	uploader.upload(obj, serial, tw, sb)
+	stopSpinner()
 
 def butSave(widget, data=None):
 	b = getCurrentPage()
@@ -268,7 +278,7 @@ def menuUpload(widget, data=None):
 	upload(widget, ser, data)
 
 def menuReference(widget, data=None):
-	gtk.show_uri(None, "file:///"+os.path.join(misc.getArduinoFile("reference"), "index.html"), int(time.time()))
+	misc.launch_in_browser(os.path.join(misc.getArduinoFile("reference"), "index.html"))
 
 def about(widget, data=None):
 	about = gui.get_object("about")
@@ -284,8 +294,10 @@ def preferences(widget, data=None):
 	fe = gui.get_object("fontbutton1")
 	fc = gui.get_object("fontbutton2")
 	bv = gui.get_object("build.verbose")
+	uv = gui.get_object("upload.verbose")
 	sn = gui.get_object("show.numbers")
 	ul = gui.get_object("user.library")
+	sd = gui.get_object("sketchdir")
 	p = prefs.preferences()
 	fe.set_font_name(misc.merge_font_name(fe, config.cur_editor_font))
 	fc.set_font_name(misc.merge_font_name(fc, config.cur_console_font))
@@ -298,6 +310,14 @@ def preferences(widget, data=None):
 			val = 1
 	bv.set_active(val)
 	val = 0
+	if config.upload_verbose != -1:
+		if config.upload_verbose == 'true':
+			val = 1
+	else:
+		if p.getSafeValue("upload.verbose", "false") == 'true':
+			val = 1
+	uv.set_active(val)
+	val = 0
 	if config.show_numbers != -1:
 		if config.show_numbers == 'true':
 			val = 1
@@ -309,6 +329,11 @@ def preferences(widget, data=None):
 		ul.set_text(str(config.user_library))
 	else:
 		ul.set_text(str(p.getSafeValue("user.library", "")))
+
+	if (config.sketchdir != None and config.sketchdir != -1):
+		sd.set_filename(str(config.sketchdir))
+	else:
+		sd.set_filename(str(p.getSafeValue("sketchbook.path", "")))
 	r = pref.run()
 	if r == 1:
 		config.cur_editor_font = fe.get_font_name()
@@ -321,6 +346,11 @@ def preferences(widget, data=None):
 		else:
 			config.build_verbose = 'false'
 		p.setValue("build.verbose", config.build_verbose)
+		if uv.get_active() == 1:
+			config.upload_verbose = 'true'
+		else:
+			config.upload_verbose = 'false'
+		p.setValue("upload.verbose", config.upload_verbose)
 		if sn.get_active() == 1:
 			config.show_numbers = 'true'
 		else:
@@ -333,6 +363,7 @@ def preferences(widget, data=None):
 			else:
 				sv.set_show_line_numbers(False)
 		p.setValue("user.library", config.user_library)
+		p.setValue("sketchbook.path", sd.get_filename())
 		p.saveValues()
 		misc.set_widget_font(tw, config.cur_console_font)
 		misc.set_widget_font(sctw, config.cur_console_font)
@@ -367,8 +398,10 @@ def cserial(w, st, data=None):
 		vbox.remove(scon)
 		vbox.add(con)
 
-def burnBootloader(w, id):
-	uploader.burnBootloader(ser, tw, sb, id)
+def burnBootloader(w):
+	startSpinner()
+	uploader.burnBootloader(ser, tw, sb, pgm.getProgrammer())
+	stopSpinner()
 
 def setBaud(w, data=None):
 	if ser.serial.isOpen():
@@ -402,22 +435,19 @@ def redo(w, data=None):
 def cut(w, data=None):
 	page = getCurrentPage()
 	b = page.get_data("buffer")
-	display = gtk.gdk.display_manager_get().get_default_display()
-	clipboard = gtk.Clipboard(display, "PRIMARY")
+	clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
 	b.cut_clipboard(clipboard, True)
 
 def copy(w, data=None):
 	page = getCurrentPage()
 	b = page.get_data("buffer")
-	display = gtk.gdk.display_manager_get().get_default_display()
-	clipboard = gtk.Clipboard(display, "PRIMARY")
+	clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
 	b.copy_clipboard(clipboard)
 
 def paste(w, data=None):
 	page = getCurrentPage()
 	b = page.get_data("buffer")
-	display = gtk.gdk.display_manager_get().get_default_display()
-	clipboard = gtk.Clipboard(display, "PRIMARY")
+	clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
 	b.paste_clipboard(clipboard, None, True)
 
 menus = [
@@ -451,6 +481,52 @@ def menu(gui):
 	gui.get_object("menu-find-next").set_sensitive(False)
 	gui.get_object("menu-replace").set_sensitive(False)
 
+def setupSpinner():
+	if gtk.pygtk_version >= (2,22,00):
+		global spinner
+		stbox = gui.get_object("statusbox")
+		spinner = gtk.Spinner()
+		stbox.pack_start(spinner, False, True)
+		stbox.reorder_child(spinner, 0)
+		context = sb.get_context_id("main")
+		spinner.set_no_show_all(True)
+
+def startSpinner():
+	if gtk.pygtk_version >= (2,22,00):
+		spinner.show()
+		spinner.start()
+
+def stopSpinner():
+	if gtk.pygtk_version >= (2,22,00):
+		spinner.stop()
+		spinner.hide()
+
+
+def addRecentItem(manager, f):
+	uri = "file://" + urllib.pathname2url(f)
+	mime = misc.get_mime_type(file(f).read())
+	if mime:
+		data = { 'mime_type': mime, 'app_name' : "gnoduino", 'app_exec' : "gnoduino" }
+		manager.add_full(uri, data)
+	else:
+		manager.add_item(uri)
+
+
+def recentMenuActivated(widget):
+	processFile(urlparse.urlparse(widget.get_current_uri()).path)
+
+def createRecentMenu():
+	menuRecent = gtk.RecentChooserMenu(recentmanager)
+	menuRecent.set_limit(10)
+	menuRecent.set_sort_type(gtk.RECENT_SORT_MRU)
+	filter = gtk.RecentFilter()
+	filter.add_application ("gnoduino");
+	menuRecent.set_filter(filter)
+	mi = gtk.MenuItem(_("Open Recent"), use_underline=True)
+	mi.set_submenu(menuRecent)
+	gui.get_object("filemenu").insert(mi, 2)
+	menuRecent.connect ("item_activated", recentMenuActivated);
+
 def createCon():
 	sw = gtk.ScrolledWindow()
 	sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
@@ -481,13 +557,14 @@ def createScon():
 	misc.set_widget_font(tmp, config.cur_console_font)
 	sw.add(tmp)
 	hbox = gtk.HBox(False, 0)
-	s = gtk.Button("Send")
-	c = gtk.Button("Clear")
+	s = gtk.Button(_("Send"))
+	c = gtk.Button(_("Clear"))
 	c.connect("clicked", ser.clearConsole, tmp)
 	bb = gtk.HBox(False, 0)
 	gui.set_data("baudHBox", bb)
 	text = gtk.Entry()
 	text.connect("activate", serSendText)
+	s.connect("clicked", serSendTextButton, text)
 	hbox.pack_start(bb, False, False, 3)
 	hbox.pack_start(text, True, True, 3)
 	hbox.pack_start(s, False, False, 3)
@@ -499,15 +576,17 @@ def createScon():
 	createBaudCombo(None, config.cur_serial_port)
 	return (vbox, tmp)
 
+def serSendTextButton(w, data=None):
+	serSendText(data, None)
+
 buttons = [
-		("compile", "compile.png", compile),
-		("stop", "stop.png", stop),
+		("verify", "verify.png", compile),
+		("upload", "upload.png", None),
 
 		("open", "open.png", copen),
 		("new", "new.png", cnew),
-
 		("save", "save.png", butSave),
-		("upload", "upload.png", None),
+
 		("serial", "serial.png", None)
 	]
 
@@ -517,7 +596,7 @@ def createBaudCombo(w, port):
 		bw = gui.get_data("baudWidget")
 		if bw: bw.destroy()
 		b = gtk.combo_box_new_text()
-		[b.append_text(i+" baud") for i in ser.getBaudrates()]
+		[b.append_text(_("%s baud") % str(i)) for i in ser.getBaudrates()]
 		b.connect("changed", setBaud)
 		b.set_active(ser.getBaudrates().index(config.serial_baud_rate))
 		hb.pack_start(b, False, False, 3)
@@ -527,6 +606,9 @@ def createBaudCombo(w, port):
 def selectBoard(w, id):
 	b.setBoard(id)
 
+def selectProgrammer(w, id):
+	pgm.setProgrammer(id)
+
 def setSerial(w, id):
 	createBaudCombo(w, id)
 	try:
@@ -564,6 +646,78 @@ def vbox_move_handle(widget, scrolltype):
 	p.saveValues()
 	return True
 
+def _search_locales():
+	localedirs = ("/usr/share/locale",
+					"/usr/local/share/locale")
+	for dir in localedirs:
+		if os.path.exists(os.path.join(dir, 'fr', 'LC_MESSAGES', 'gnoduino.mo')):
+			return dir
+    # fall back to the current directory
+	return "locale/"
+
+def exampleProcess(widget):
+	processFile(widget.get_data("file"))
+
+def populateExampleLine(entry, menu):
+	subitem = gtk.Menu()
+	menuItem = gtk.MenuItem(os.path.basename(entry))
+	ext = False
+	for i in sorted(os.listdir(entry)):
+		if os.path.isdir(os.path.join(entry,i)):
+			f = os.path.join(entry, i, i + ".ino")
+			if not os.path.exists(f):
+				f = os.path.join(entry, i, i + ".pde")
+				if not os.path.exists(f): continue
+			ext = True
+			item = gtk.MenuItem(os.path.basename(i))
+			item.set_data("file", f)
+			item.connect("activate", exampleProcess)
+			subitem.append(item)
+		else:
+			d = os.path.join(entry, "examples")
+			if os.path.exists(d):
+				for j in sorted(os.listdir(d)):
+					f = os.path.join(entry, "examples", j, j + ".ino")
+					if not os.path.exists(f):
+						f = os.path.join(entry, "examples", j, j + ".pde")
+						if not os.path.exists(f): continue
+					ext = True
+					item = gtk.MenuItem(os.path.basename(j))
+					item.set_data("file", f)
+					item.connect("activate", exampleProcess)
+					subitem.append(item)
+				break
+	if ext:
+		menuItem.set_submenu(subitem)
+		menu.append(menuItem)
+	else:
+		if os.path.basename(os.path.split(entry)[0]) == "examples":
+			menuItem.set_data("file", os.path.join(entry, i))
+			menuItem.connect("activate", exampleProcess)
+			menu.append(menuItem)
+
+def populateExamples():
+	submenu = gtk.Menu()
+	for dir in ["examples", "libraries"]:
+		if misc.get_path(dir, "\0") != "\0":
+			d =  os.listdir(misc.get_path(dir))
+		else: continue
+		q = []
+		for i in d: q.append(misc.get_path(os.path.join(dir, i)))
+		for c in sorted(q): populateExampleLine(c, submenu)
+	paths = []
+	if config.user_library != None and config.user_library != -1:
+		paths.extend(i.strip() for i in config.user_library.split(';'))
+	for p in paths:
+		if os.path.exists(p):
+			q = []
+			if os.path.isdir(os.path.join(p, "examples")):
+				q.append(p)
+		for c in sorted(q): populateExampleLine(c, submenu)
+	ex = gtk.MenuItem(_("E_xamples"), use_underline=True)
+	ex.set_submenu(submenu)
+	gui.get_object("filemenu").insert(ex, 2)
+
 def run():
 	try:
 		global gui
@@ -581,30 +735,39 @@ def run():
 		global scon
 		global p
 		global b
+		global pgm
+		global recentmanager
+		locale_path = _search_locales()
+		gettext.bindtextdomain(APP_NAME, locale_path)
+		gettext.textdomain(APP_NAME)
+		locale.bindtextdomain(APP_NAME, locale_path)
+		#global spinner
 		#perform cleanup prior to this
 		signal.signal(signal.SIGINT, signal.SIG_DFL)
 		id = misc.makeWorkdir()
 		ser = serialio.sconsole()
 		p = prefs.preferences()
+		recentmanager = gtk.recent_manager_get_default()
 		sertime = None
 		gui = gtk.Builder()
+		gui.set_translation_domain(APP_NAME)
 		try:
 			path = os.path.join(os.getcwd(), "ui", "main.ui")
 			if os.path.exists(path):
 				gui.add_from_file(path)
-			else: raise NameError("System error")
+			else: raise NameError(_("System error"))
 		except:
 			try:
 				path = os.path.join(sys.prefix, "local", "share", "gnoduino", "ui", "main.ui")
 				if os.path.exists(path):
 					gui.add_from_file(path)
-				else: raise NameError("System error")
+				else: raise NameError(_("System error"))
 			except:
 				try:
 					path = os.path.join(sys.prefix, "share", "gnoduino", "ui", "main.ui")
 					if os.path.exists(path):
 						gui.add_from_file(path)
-					else: raise NameError("System error")
+					else: raise NameError(_("System error"))
 				except Exception,e:
 					print(e)
 					raise SystemExit(_("Cannot load ui file"))
@@ -617,7 +780,10 @@ def run():
 		mainwin.connect("configure-event", cb_configure_event)
 		vbox = gui.get_object("vpan")
 		sb = gui.get_object("statusbar1")
+		sb.set_has_resize_grip(False)
 		sb2 = gui.get_object("statusbar2")
+		sb2.set_has_resize_grip(False)
+		setupSpinner()
 		config.cur_editor_font = p.getSafeValue("editor.font", p.getDefaultValue("editor.font")).replace(",", " ")
 		config.cur_console_font = p.getSafeValue("console.font", p.getDefaultValue("console.font")).replace(",", " ")
 		config.build_verbose = p.getSafeValue("build.verbose", p.getDefaultValue("build.verbose"))
@@ -627,6 +793,7 @@ def run():
 		"""build menus"""
 		sub = gtk.Menu()
 		b = board.Board()
+		pgm = programmer.Programmer()
 		maingroup = gtk.RadioMenuItem(None, None)
 		for i in b.getBoards():
 			menuItem = gtk.RadioMenuItem(maingroup, i['desc'])
@@ -675,14 +842,19 @@ def run():
 
 		gui.get_object("serial_port").set_submenu(sub)
 		gui.get_object("serial_port").set_sensitive(activePort)
+		createRecentMenu()
+		populateExamples()
 
 		sub = gtk.Menu()
-		pgm = programmer.Programmer()
+		maingroup = gtk.RadioMenuItem(None, None)
 		for i in pgm.getProgrammers():
-			menuItem = gtk.MenuItem(i['desc'])
-			menuItem.connect('activate', burnBootloader, i['id'])
+			menuItem = gtk.RadioMenuItem(maingroup, i['desc'])
+			if i['id'] == pgm.getProgrammer() + 1:
+				menuItem.set_active(True)
+			menuItem.connect('activate', selectProgrammer, i['id'])
 			sub.append(menuItem)
-		gui.get_object("burn").set_submenu(sub)
+		gui.get_object("programmer").set_submenu(sub)
+		gui.get_object("burn").connect('activate', burnBootloader)
 
 		nb = gtk.Notebook()
 		nb.connect("switch-page", setupPage)
@@ -693,6 +865,7 @@ def run():
 		vbox.connect("notify::position", vbox_move_handle)
 		cpos = int(p.getSafeValue("console.height", -1))
 		vbox.set_position(cpos)
+
 		mainwin.set_focus(sv)
 		mainwin.show_all()
 		mainwin.set_title("Gnoduino")
@@ -710,7 +883,8 @@ def run():
 				o.connect("clicked", i[2])
 		if (sys.argv[1:]):
 			if sys.argv[1:][0] == "--help" or sys.argv[1:][0] == "-h":
-				print """--help Prints the command line options.\n--version Output version information and exit"""
+				print _("--help    Print the command line options")
+				print _("--version Output version information and exit")
 				sys.exit(0)
 			if sys.argv[1:][0] == "--version" or sys.argv[1:][0] == "-v":
 				print "gnoduino %s" % gnoduino.__version__
