@@ -1,5 +1,5 @@
 # gnoduino - Python Arduino IDE implementation
-# Copyright (C) 2010  Lucian Langa
+# Copyright (C) 2010-2012  Lucian Langa
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -60,40 +60,65 @@ def burnBootloader(serial, output, notify, id):
 	"""De-fuse and erase board"""
 	compline=[i for i in avr_bl]
 	compline.append("-c" + pgm.getProtocol(id))
+	if pgm.getCommunication(id) == 'serial':
+		port = serial.getConfigSerialPort(notify, output)
+		if port == -1:
+			notify.pop(context)
+			notify.push(context, _("Flashing error."))
+			return
+		compline.append("-P" + port)
+		if pgm.getSpeed(id) != 0:
+			compline.append("-b" + pgm.getSpeed(id))
+	elif pgm.getCommunication(id) == 'usb':
+		compline.append("-Pusb")
 	compline.append("-p" + b.getBoardMCU(b.getBoard()))
 	compline.append("-e")
 	if pgm.getForce(id) == 'true':
 		compline.append("-F")
 	compline.append("-Ulock:w:" + b.getFuseUnlock(b.getBoard()) + ":m")
-	compline.append("-Uefuse:w:" + b.getFuseExtended(b.getBoard()) + ":m")
+	if b.getBoardMCU(b.getBoard()) != 'atmega8':
+		compline.append("-Uefuse:w:" + b.getFuseExtended(b.getBoard()) + ":m")
 	compline.append("-Uhfuse:w:" + b.getFuseHigh(b.getBoard()) + ":m")
 	compline.append("-Ulfuse:w:" + b.getFuseLow(b.getBoard()) + ":m")
-	misc.printMessageLn(output, ' '.join(compline))
 	try:
+		if p.getBoolValue("upload.verbose"):
+			sys.stderr.write(' '.join(compline)+"\n")
+			misc.printMessageLn(output, ' '.join(compline))
 		(run, sout) = misc.runProg(compline)
-		if run == False:
-			if p.getBoolValue("build.verbose"):
-				misc.printErrorLn(notify, output, _("Burn Error"), sout)
-			raise
+		misc.printMessageLn(output, sout, p.getBoolValue("upload.verbose"), 'false')
+		if p.getBoolValue("upload.verbose"): sys.stderr.write(sout+"\n")
+		if run == False: raise
 	except:
 		misc.printErrorLn(notify, output, _("Burn Error"), _("Burn ERROR."))
 		return
 	"""Burn and fuse board"""
 	compline=[i for i in avr_bl]
 	compline.append("-c" + pgm.getProtocol(id))
+	if pgm.getCommunication(id) == 'serial':
+		port = serial.getConfigSerialPort(notify, output)
+		if port == -1:
+			notify.pop(context)
+			notify.push(context, _("Flashing error."))
+			return
+		compline.append("-P" + port)
+		if pgm.getSpeed(id) != 0:
+			compline.append("-b" + pgm.getSpeed(id))
+	elif pgm.getCommunication(id) == 'usb':
+		compline.append("-Pusb")
 	compline.append("-p" + b.getBoardMCU(b.getBoard()))
 	compline.append("-e")
 	if pgm.getForce(id) == 'true':
 		compline.append("-F")
 	compline.append("-Uflash:w:" + findBootLoader() + ":i")
 	compline.append("-Ulock:w:" + b.getFuseLock(b.getBoard()) + ":m")
-	if p.getBoolValue("build.verbose"): print compline
 	try:
-		run = misc.runProgOutput(output, compline)
-		if run == False:
-			if p.getBoolValue("build.verbose"):
-				misc.printErrorLn(notify, output, _("Burn Error"), sout)
-			raise
+		if p.getBoolValue("upload.verbose"):
+			sys.stderr.write(' '.join(compline)+"\n")
+			misc.printMessageLn(output, ' '.join(compline))
+		(run, sout) = misc.runProg(compline)
+		misc.printMessageLn(output, sout, p.getBoolValue("upload.verbose"), 'false')
+		if p.getBoolValue("upload.verbose"): sys.stderr.write(sout+"\n")
+		if run == False: raise
 	except:
 		misc.printErrorLn(notify, output, _("Burn Error"), _("Burn ERROR."))
 		return
@@ -104,31 +129,53 @@ def burnBootloader(serial, output, notify, id):
 
 def upload(obj, serial, output, notify):
 	p = prefs.preferences()
+	pgm = programmer.Programmer()
 	context = notify.get_context_id("main")
 	notify.pop(context)
 	notify.push(context, _("Flashing..."))
 	b = board.Board()
-	port = serial.getConfigSerialPort(notify, output)
-	if port == -1:
-		notify.pop(context)
-		notify.push(context, _("Flashing error."))
-		return
-	serial.resetBoard()
 	compline=[i for i in avr]
-	# avrdude wants "stk500v1" to distinguish it from stk500v2
 	protocol = b.getPGM(b.getBoard())
+	# avrdude wants "stk500v1" to distinguish it from stk500v2
 	if protocol == "stk500": protocol = "stk500v1"
+	if protocol == "":
+		protocol =  pgm.getProtocol(pgm.getProgrammer())
+		try:
+			comm = pgm.getCommunication(pgm.getProgrammer())
+			if comm == "serial":
+				port = serial.getConfigSerialPort(notify, output)
+				if port == -1:
+					notify.pop(context)
+					notify.push(context, _("Flashing error."))
+					return
+				serial.resetBoard()
+				compline.append("-P" + port)
+			else: compline.append("-P" + comm)
+			try:
+				compline.append("-b" + pgm.getSpeed(pgm.getProgrammer()))
+			except: pass
+		except: pass
+	else:
+		port = serial.getConfigSerialPort(notify, output)
+		if port == -1:
+			notify.pop(context)
+			notify.push(context, _("Flashing error."))
+			return
+		compline.append("-P" + port)
+		serial.resetBoard()
 	compline.append("-c" + protocol)
-	compline.append("-P" + port)
-	compline.append("-b" + b.getPGMSpeed(b.getBoard()))
+	try:
+		compline.append("-b" + b.getPGMSpeed(b.getBoard()))
+	except: pass
 	compline.append("-p" + b.getBoardMCU(b.getBoard()))
 	compline.append("-Uflash:w:"+obj+".hex:i")
 	try:
-		if p.getBoolValue("build.verbose"):
+		if p.getBoolValue("upload.verbose"):
 			sys.stderr.write(' '.join(compline)+"\n")
 			misc.printMessageLn(output, ' '.join(compline))
 		(run, sout) = misc.runProg(compline)
-		misc.printMessageLn(output, sout, p.getBoolValue("build.verbose"), 'false')
+		misc.printMessageLn(output, sout, p.getBoolValue("upload.verbose"), 'false')
+		if p.getBoolValue("upload.verbose"): sys.stderr.write(sout+"\n")
 		if run == False: raise
 	except:
 		misc.printErrorLn(notify, output, _("Flashing Error"), _("Flash ERROR.\n"))
@@ -140,6 +187,8 @@ def upload(obj, serial, output, notify):
 
 def findBootLoader():
 	b = board.Board()
-	return os.path.join(misc.getArduinoBootPath(), b.getPath(b.getBoard()), \
-		b.getBootloader(b.getBoard())+".hex")
+	boot = b.getBootloader(b.getBoard())
+	if boot.endswith(".hex"): None
+	else: boot = boot + ".hex"
+	return os.path.join(misc.getArduinoBootPath(), b.getPath(b.getBoard()), boot)
 
